-- Wallet is a simple money and/or destiny point display

import "GaranStuff.Wallet.Class";
import "GaranStuff.Wallet.Table";
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "GaranStuff.Wallet.VindarPatch";

euroFormat=(tonumber("1,000")==1);
walletShowMoney=true;
walletShowDP=true;
walletTop=0;
walletLeft=0;
walletOpacity=.5;

walletHUDState=true;
walletMainState=true;
walletMoveToggle=false;
localPlayer=Turbine.Gameplay.LocalPlayer:GetInstance();

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

LoadSettings=function()
	local settings=PatchDataLoad( Turbine.DataScope.Character, "Wallet_Settings");
	if settings~=nil then
		local tmpIndex;
		for tmpIndex=1,#settings do
			if settings[tmpIndex][1]=="ShowMoney" then
				walletShowMoney=settings[tmpIndex][2];
			elseif settings[tmpIndex][1]=="ShowDP" then
				walletShowDP=settings[tmpIndex][2];
			elseif settings[tmpIndex][1]=="Top" then
				walletTop=settings[tmpIndex][2];
			elseif settings[tmpIndex][1]=="Left" then
				walletLeft=settings[tmpIndex][2];
			elseif settings[tmpIndex][1]=="Opacity" then
				if euroFormat then
					-- replace "." with "," tonumber(string.gsub(tmpSettings[index][2].R,".",","))
					walletOpacity=tonumber((string.gsub(settings[tmpIndex][2],"%.",",")));
				else
					walletOpacity=tonumber((string.gsub(settings[tmpIndex][2],",",".")));
				end
			end
		end
	end
end
LoadSettings()

Wallet=class(Turbine.UI.Window);

function Wallet:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:SetSize(173,60);
	self:SetPosition(walletLeft,walletTop);
	self:SetBackColor(Turbine.UI.Color(0,0,0,0)); -- a necessary evil since RoI for some reason. if we don't include this, the window doesn't redraw properly
	self:SetOpacity(walletOpacity);

	self.MovePanel=Turbine.UI.Control();
	self.MovePanel:SetParent(self);
	self.MovePanel:SetSize(173,19);
	self.MovePanel:SetBackColor(Turbine.UI.Color(.5,0,0,0));
	self.Move2=Turbine.UI.Control();
	self.Move2:SetParent(self.MovePanel);
	self.Move2:SetSize(156,12);
	self.Move2:SetPosition(17,2);
	self.Move2:SetBackground("GaranStuff/Wallet/Resources/move2.tga");
	self.Move2:SetMouseVisible(false);
	self.Move=Turbine.UI.Control();
	self.Move:SetParent(self.MovePanel);
	self.Move:SetSize(19,19);
	self.Move:SetBackground("GaranStuff/Wallet/Resources/move.tga");
	self.Move:SetMouseVisible(false);
	self.MoveText=Turbine.UI.Label();
	self.MoveText:SetParent(self.MovePanel);
	self.MoveText:SetSize(156,12);
	self.MoveText:SetPosition(17,2);
	self.MoveText:SetFont(Turbine.UI.Lotro.Font.Verdana10);
	self.MoveText:SetText("Wallet");
	self.MoveText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.MoveText:SetMouseVisible(false);
	self.MovePanel:SetVisible(false);

	self.MoveX=-1;
	self.MoveY=0;
	self.Moving=false;

	self.MovePanel.MouseDown=function()
	end

	self.MovePanel.MouseClick = function(sender,args)
--		self.Pin:SetBackground("GaranStuff/Rainbow/Resources/icon_up.tga");
		if self.Moving then
			self.Moving=false;
			self.MoveX=-1;
		end
	end
	self.MovePanel.MouseDown=function(sender,args)
--		self.Pin:SetBackground("GaranStuff/Rainbow/Resources/icon_dn.tga");
Turbine.Shell.WriteLine("mouse down")
		self.MoveX=args.X;
		self.MoveY=args.Y;
	end
	self.MovePanel.MouseUp=function(sender,args)
--		self.Pin:SetBackground("GaranStuff/Rainbow/Resources/icon_up.tga");
		self.MoveX=-1;
		self.MoveY=0;
	end

	self.MovePanel.MouseEnter=function(sender,args)
		self:SetOpacity(1);
	end

	self.MovePanel.MouseLeave=function(sender,args)
		self:SetOpacity(walletOpacity);
	end
	self.MovePanel.MouseMove=function(sender,args)
		if self.MoveX~=-1 and (args.X~=self.MoveX or args.Y~= self.MoveY) then
--			self.Pin:SetBackground("GaranStuff/Rainbow/Resources/icon_drag.tga");
			self.Moving=true;
			local newLeft=self:GetLeft()-(self.MoveX-args.X)
			local newTop=self:GetTop()-(self.MoveY-args.Y)
			if newLeft<0 then newLeft=0 end;
			if newLeft>(Turbine.UI.Display.GetWidth()-self:GetWidth()) then newLeft=Turbine.UI.Display.GetWidth()-self:GetWidth() end;
			if newTop<0 then newTop=0 end;
			if newTop>(Turbine.UI.Display.GetHeight()-self:GetHeight()) then newTop=Turbine.UI.Display.GetHeight()-self:GetHeight() end;
			self:SetPosition(newLeft,newTop);
		end
	end

	self.WalletPanel=Turbine.UI.Label();
	self.WalletPanel:SetParent(self);
	self.WalletPanel:SetBackColor(Turbine.UI.Color(.3,0,0,0));
	self.WalletPanel:SetPosition(0,20);
	self.WalletPanel:SetSize(self:GetWidth(),40);
	self.WalletPanel:SetMouseVisible(false);
	self.GPText=Turbine.UI.Label();
	self.GPText:SetParent(self.WalletPanel);
	self.GPText:SetSize(30,20);
	self.GPText:SetPosition(0,0);
	self.GPText:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.GPText:SetText("");
	self.GPText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.GPText:SetMouseVisible(false);
	self.GPText:SetVisible(walletShowMoney);
	self.GPIcon=Turbine.UI.Control();
	self.GPIcon:SetParent(self.WalletPanel);
	self.GPIcon:SetSize(27,21);
	self.GPIcon:SetPosition(30,0);
	self.GPIcon:SetBackground(0x41007e7b);
	self.GPIcon:SetMouseVisible(false);
	self.GPIcon:SetVisible(walletShowMoney);

	self.SPText=Turbine.UI.Label();
	self.SPText:SetParent(self.WalletPanel);
	self.SPText:SetSize(30,20);
	self.SPText:SetPosition(62,0);
	self.SPText:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.SPText:SetText("");
	self.SPText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.SPText:SetMouseVisible(false);
	self.SPText:SetVisible(walletShowMoney);
	self.SPIcon=Turbine.UI.Control();
	self.SPIcon:SetParent(self.WalletPanel);
	self.SPIcon:SetSize(27,21);
	self.SPIcon:SetPosition(92,0);
	self.SPIcon:SetBackground(0x41007e7c);
	self.SPIcon:SetMouseVisible(false);
	self.SPIcon:SetVisible(walletShowMoney);

	self.CPText=Turbine.UI.Label();
	self.CPText:SetParent(self.WalletPanel);
	self.CPText:SetSize(22,20);
	self.CPText:SetPosition(124,0);
	self.CPText:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.CPText:SetText("");
	self.CPText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.CPText:SetMouseVisible(false);
	self.CPText:SetVisible(walletShowMoney);
	self.CPIcon=Turbine.UI.Control();
	self.CPIcon:SetParent(self.WalletPanel);
	self.CPIcon:SetSize(27,21);
	self.CPIcon:SetPosition(146,0);
	self.CPIcon:SetBackground(0x41007e7c);
	self.CPIcon:SetMouseVisible(false);
	self.CPIcon:SetVisible(walletShowMoney);

	self.DP=Turbine.UI.Label();
	self.DP:SetParent(self.WalletPanel);
	self.DP:SetSize(146,20);
	self.DP:SetPosition(0,20);
	self.DP:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.DP:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.DP:SetVisible(walletShowDP);
	self.DP:SetMouseVisible(false);
	self.DPIcon=Turbine.UI.Control();
	self.DPIcon:SetParent(self.WalletPanel);
	self.DPIcon:SetSize(21,20);
	self.DPIcon:SetPosition(149,19);
	self.DPIcon:SetBackground(0x4100a682);
	self.DPIcon:SetMouseVisible(false);


	self:SetMouseVisible(false);

	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("Wallet",self.shellCmd);

	self.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "wallet") then
			local pos1=string.find(args," ",0,true);
			local command;
			if pos1~=nil and pos1>0 then
				command=string.lower(string.sub(args,1,pos1-1));
			else
				command=string.lower(args);
			end
			if (command=="show") then
				walletWindow:SetVisible(true);
			elseif (command=="hide") then
				walletWindow:SetVisible(false);
			elseif (command=="toggle") then
				walletWindow:SetVisible(not walletWindow:IsVisible());
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /Wallet show");
				Turbine.Shell.WriteLine("       Displays the Wallet.");
				Turbine.Shell.WriteLine("  /Wallet hide");
				Turbine.Shell.WriteLine("       Hides the Wallet.");
				Turbine.Shell.WriteLine("  /Wallet toggle");
				Turbine.Shell.WriteLine("       Toggles the Wallet (shows it if it was hidden, hides it if it was showing).");
			end
		end
	end

	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /Wallet show\n       Displays the Wallet.\n  /Wallet hide\n       Hides the Wallet.\n  /Walet toggle\n       Toggles the Wallet (shows it if it was hidden, hides it if it was showing).");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("Wallet - usage: /Wallet show | hide | toggle");
	end

	self.loaded=false;
	self:SetWantsUpdates(true);
	self:SetWantsKeyEvents(true);
	self.Changed=true;

	self.Update=function()
		if self.Changed then
			local money=self.playerAttributes:GetMoney();
			local cp=math.fmod(money,100);
			money=(money-cp)/100;
			local sp=math.fmod(money,1000);
			local gp=(money-sp)/1000;
			self.GPText:SetText(gp);
			self.SPText:SetText(sp);
			self.CPText:SetText(cp);

			self.DP:SetText(self.playerAttributes:GetDestinyPoints());
			self.Changed=false;
		end
		if not self.loaded then
			self.loaded=true;
			Plugins["Wallet"].Unload = function(self,sender,args)
				UnloadWallet();
			end
			self:SetWantsUpdates(false);
		end
	end
	self.KeyDown=function(sender, args)
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			walletWindow:SetVisible(false);
		end

		if ( args.Action == 268435579 ) then -- reposition UI
			walletMoveToggle=not walletMoveToggle;
			walletWindow.MovePanel:SetVisible(walletMoveToggle);
		end

		if ( args.Action == 268435635 ) then -- toggle HUD
			walletHUDState = not walletHUDState;
			if walletHUDState then
				walletWindow:SetVisible(walletMainState);
			else
				walletMainState=walletWindow:IsVisible();
				walletWindow:SetVisible(false);
			end
		end
	end
	self.playerAttributes=localPlayer:GetAttributes();

	self.moneyChangedHandler = AddCallback(self.playerAttributes, "MoneyChanged", function(sender, args) self.Changed=true; self:SetWantsUpdates(true); end)
	self.dpChangedHandler = AddCallback(self.playerAttributes, "DestinyPointsChanged", function(sender, args) self.Changed=true; self:SetWantsUpdates(true); end)

end
function UnloadWallet()
	-- release the event handlers
	RemoveCallback(walletWindow.playerAttributes, "MoneyChanged", walletWindow.moneyChangedHandler);
	RemoveCallback(walletWindow.playerAttributes, "DestinyPointsChanged", walletWindow.dpChangedHandler);

	Turbine.Shell.WriteLine("Wallet "..Plugins["Wallet"]:GetVersion().." by Garan unloaded");

	local Settings={};
	Settings[#Settings+1]={"ShowMoney",walletShowMoney};
	Settings[#Settings+1]={"ShowDP",walletShowDP};
	Settings[#Settings+1]={"Top",walletWindow:GetTop()};
	Settings[#Settings+1]={"Left",walletWindow:GetLeft()};
	Settings[#Settings+1]={"Opacity",walletOpacity};

	PatchDataSave( Turbine.DataScope.Character, "Wallet_Settings", Settings );
end
walletWindow=Wallet()
walletWindow:SetVisible(true);
